@@ -19,10 +19,11 @@ import networkx as nx
 import matplotlib.pyplot as plt
 from pathlib import Path
 from tqdm import tqdm
+from tabulate import tabulate
 
 from pgmpy.models import BayesianNetwork, NaiveBayes
 from pgmpy.factors.discrete import TabularCPD
-from pgmpy.sampling import BayesianModelSampling  # GibbsSampling
+from pgmpy.sampling import BayesianModelSampling, GibbsSampling
 from pgmpy import readwrite
 
 from ismember import ismember
@@ -62,7 +63,7 @@ def to_bayesiannetwork(model, verbose=3):
         adjmat = model
     if adjmat is None: raise Exception('[bnlearn] >Error: input for "bayesiannetwork" should be adjmat or a dict containing a key "adjmat".')
 
-    if verbose>=3: print('[bnlearn] >Conversion of adjmat to BayesianNetwork.')
+    if verbose>=3: print('[bnlearn] >Converting adjmat to BayesianNetwork.')
 
     # Convert to vector
     vec = adjmat2vec(adjmat)[['source', 'target']].values.tolist()
@@ -133,6 +134,10 @@ def make_DAG(DAG, CPD=None, methodtype='bayes', checkmodel=True, verbose=3):
     elif isinstance(DAG, list) and methodtype=='bayes':
         if verbose>=3: print('[bnlearn] >%s DAG created.' %(methodtype))
         DAG = BayesianNetwork(DAG)
+    elif isinstance(DAG, list) and methodtype=='markov':
+        from pgmpy.models import MarkovNetwork
+        if verbose>=3: print('[bnlearn] >%s DAG created.' %(methodtype))
+        DAG = MarkovNetwork(DAG)
 
     if CPD is not None:
         for cpd in CPD:
@@ -152,7 +157,7 @@ def make_DAG(DAG, CPD=None, methodtype='bayes', checkmodel=True, verbose=3):
 
 
 # %% Print DAG
-def print_CPD(DAG, checkmodel=False):
+def print_CPD(DAG, checkmodel=False, verbose=3):
     """Print DAG-model to screen.
 
     Parameters
@@ -174,6 +179,10 @@ def print_CPD(DAG, checkmodel=False):
     # Print CPDs
     # if config['method']=='ml' or config['method']=='maximumlikelihood':
     try:
+        if ('markovnetwork' in str(type(DAG)).lower()):
+            if verbose>=3: print('[bnlearn] >Converting markovnetwork to Bayesian model')
+            DAG=DAG.to_bayesian_model()
+
         if 'MaximumLikelihood' in str(type(DAG)):
             # print CPDs using Maximum Likelihood Estimators
             for node in DAG.state_names:
@@ -184,17 +193,20 @@ def print_CPD(DAG, checkmodel=False):
                 raise Exception('[bnlearn] >Error! This is a Bayesian DAG containing only edges, and no CPDs. Tip: you need to specify or learn the CPDs. Try: DAG=bn.parameter_learning.fit(DAG, df). At this point you can make a plot with: bn.plot(DAG).')
                 return
             for cpd in DAG.get_cpds():
-                print("CPD of {variable}:".format(variable=cpd.variable))
-                print(cpd)
+                if verbose>=3:
+                    print("CPD of {variable}:".format(variable=cpd.variable))
+                    print(cpd)
             if ('bayesiannetwork' in str(type(DAG)).lower()):
-                print('[bnlearn] >Independencies:\n%s' %(DAG.get_independencies()))
-            print('[bnlearn] >Nodes: %s' %(DAG.nodes()))
-            print('[bnlearn] >Edges: %s' %(DAG.edges()))
+                if verbose>=3: print('[bnlearn] >Independencies:\n%s' %(DAG.get_independencies()))
+
+            if verbose>=3:
+                print('[bnlearn] >Nodes: %s' %(DAG.nodes()))
+                print('[bnlearn] >Edges: %s' %(DAG.edges()))
 
         if checkmodel:
             _check_model(DAG, verbose=3)
     except:
-        print('[bnlearn] >No CPDs to print. Hint: Add CPDs as following: <bn.make_DAG(DAG, CPD=[cpd_A, cpd_B, etc])> and use bnlearn.plot(DAG) to make a plot.')
+        if verbose>=2: print('[bnlearn] >No CPDs to print. Hint: Add CPDs as following: <bn.make_DAG(DAG, CPD=[cpd_A, cpd_B, etc])> and use bnlearn.plot(DAG) to make a plot.')
 
 
 # %%
@@ -433,13 +445,16 @@ def adjmat2dict(adjmat):
 
 
 # %% Sampling from model
-def sampling(DAG, n=1000, verbose=3):
-    """Generate sample(s) using forward sampling from joint distribution of the bayesian network.
+def sampling(DAG, n=1000, methodtype='bayes', verbose=0):
+    """Generate sample(s) using the joint distribution of the network.
 
     Parameters
     ----------
     DAG : dict
-        Contains model and adjmat of the DAG.
+        Contains model and the adjmat of the DAG.
+    methodtype : str (default: 'bayes')
+        * 'bayes': Forward sampling using Bayesian.
+        * 'gibbs' : Gibbs sampling.
     n : int, optional
         Number of samples to generate. The default is 1000.
     verbose : int, optional
@@ -451,27 +466,54 @@ def sampling(DAG, n=1000, verbose=3):
     df : pd.DataFrame().
         Dataframe containing sampled data from the input DAG model.
 
-
     Example
     -------
+    >>> # Example 1
+    >>>
+    >>> # Import library
     >>> import bnlearn as bn
+    >>> # Load DAG with model
     >>> DAG = bn.import_DAG('sprinkler')
-    >>> df = bn.sampling(DAG, n=1000)
+    >>> # Sampling
+    >>> df = bn.sampling(DAG, n=1000, methodtype='bayes')
+    >>>
+    >>> # Example 2:
+    >>>
+    >>> # Load example dataset
+    >>> df = bn.import_example('sprinkler')
+    >>> edges = [('Cloudy', 'Sprinkler'),
+    >>>         ('Cloudy', 'Rain'),
+    >>>         ('Sprinkler', 'Wet_Grass'),
+    >>>         ('Rain', 'Wet_Grass')]
+    >>>
+    >>> # Make the actual Bayesian DAG
+    >>> DAG = bn.make_DAG(edges, verbose=3, methodtype='bayes')
+    >>> # Fit model
+    >>> model = bn.parameter_learning.fit(DAG, df, verbose=3, methodtype='bayes')
+    >>> # Sampling using gibbs
+    >>> df = bn.sampling(model, n=100, methodtype='gibbs', verbose=0)
 
     """
-    if n<=0: raise ValueError('n must be 1 or larger')
-    if 'BayesianNetwork' not in str(type(DAG['model'])): raise ValueError('DAG must contain BayesianNetwork.')
-    if verbose>=3: print('[bnlearn] >Forward sampling for %.0d samples..' %(n))
+    if n<=0: raise ValueError('Number of samples (n) must be 1 or larger!')
+    if (DAG is None) or ('BayesianNetwork' not in str(type(DAG['model']))):
+        raise ValueError('The input model (DAG) must contain BayesianNetwork.')
 
     if len(DAG['model'].get_cpds())==0:
         raise Exception('[bnlearn] >Error! This is a Bayesian DAG containing only edges, and no CPDs. Tip: you need to specify or learn the CPDs. Try: DAG=bn.parameter_learning.fit(DAG, df). At this point you can make a plot with: bn.plot(DAG).')
         return
 
-    # http://pgmpy.org/sampling.html
-    infer_model = BayesianModelSampling(DAG['model'])
-    # inference1 = GibbsSampling(model['model'])
-    # Forward sampling and make dataframe
-    df=infer_model.forward_sample(size=n)
+    if methodtype=='bayes':
+        if verbose>=3: print('[bnlearn] >Bayesian forward sampling for %.0d samples..' %(n))
+        # Bayesian Forward sampling and make dataframe
+        infer_model = BayesianModelSampling(DAG['model'])
+        df = infer_model.forward_sample(size=n, seed=None, show_progress=(True if verbose>=3 else False))
+    elif methodtype=='gibbs':
+        if verbose>=3: print('[bnlearn] >Gibbs sampling for %.0d samples..' %(n))
+        # Gibbs sampling
+        gibbs = GibbsSampling(DAG['model'])
+        df = gibbs.sample(size=n, seed=None)
+    else:
+        if verbose>=3: print('[bnlearn] >Methodtype [%s] unknown' %(methodtype))
     return(df)
 
 
@@ -517,7 +559,33 @@ def _bif2bayesian(pathname, verbose=3):
 
 
 # %%
-def query2df(query, variables=None):
+# def query2df(query, variables=None):
+#     """Convert query from inference model to a dataframe.
+
+#     Parameters
+#     ----------
+#     query : Object from the inference model.
+#         Convert query object to a dataframe.
+#     variables : list
+#         Order or select variables.
+
+#     Returns
+#     -------
+#     df : pd.DataFrame()
+#         Dataframe with inferences.
+
+#     """
+#     df = pd.DataFrame(data=list(itertools.product(np.arange(0, len(query.values)), repeat=len(query.variables))), columns=query.variables)
+#     df['p'] = query.values.flatten()
+#     # Order or filter on input variables
+#     if variables is not None:
+#         # Add Pvalue column
+#         variables = variables + ['p']
+#         df = df[variables]
+#     return df
+
+
+def query2df(query, variables=None, verbose=3):
     """Convert query from inference model to a dataframe.
 
     Parameters
@@ -533,15 +601,26 @@ def query2df(query, variables=None):
         Dataframe with inferences.
 
     """
-    df = pd.DataFrame(data=list(itertools.product(np.arange(0, len(query.values)), repeat=len(query.variables))), columns=query.variables)
-    df['p'] = query.values.flatten()
+    states = []
+    getP = []
+    for value_index, prob in enumerate(itertools.product(*[range(card) for card in query.cardinality])):
+        states.append(prob)
+        getP.append(query.values.ravel()[value_index])
+
+    df = pd.DataFrame(data=states, columns=query.scope())
+    df['p'] = getP
     # Order or filter on input variables
     if variables is not None:
         # Add Pvalue column
-        variables=variables +['p']
+        variables = variables + ['p']
         df = df[variables]
-    return df
 
+    # Print table to screen
+    if verbose>=3:
+        print('[bnlearn] >Data is stored in [query.df]')
+        print(tabulate(df, tablefmt="grid", headers="keys"))
+
+    return df
 
 # %% Model Sprinkler
 def _DAG_sprinkler(CPD=True):
